unit uViewRecordData;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, ComCtrls, Buttons, ClipBrd, FWEventLog;

type
  TdlgViewRecordData = class(TForm)
    pcEvent: TPageControl;
    tsEvent: TTabSheet;
    lblDate: TLabel;
    lblDateVal: TLabel;
    lblTime: TLabel;
    lblTimeVal: TLabel;
    lblType: TLabel;
    lblTypeVal: TLabel;
    lblUser: TLabel;
    lblUserVal: TLabel;
    lblComp: TLabel;
    lblCompVal: TLabel;
    lblSource: TLabel;
    lblSourceVal: TLabel;
    lblCategory: TLabel;
    lblCategryVal: TLabel;
    lblCode: TLabel;
    lblCodeVal: TLabel;
    lblDescription: TLabel;
    memDescription: TMemo;
    lblData: TLabel;
    rbByte: TRadioButton;
    rbWord: TRadioButton;
    memData: TMemo;
    btnOK: TButton;
    btnUp: TBitBtn;
    btnDown: TBitBtn;
    btnCopy: TBitBtn;
    procedure btnCopyClick(Sender: TObject);
    procedure btnDownClick(Sender: TObject);
    procedure btnUpClick(Sender: TObject);
    procedure rbWordClick(Sender: TObject);
    procedure rbByteClick(Sender: TObject);
  private
    FEventRecord: TFWEventLogRecord;
  public
    procedure ShowRecData(EventRecord: TFWEventLogRecord);
  end;

var
  dlgViewRecordData: TdlgViewRecordData;

implementation

{$R *.dfm}

uses
  uMain;

{ TdlgViewRecordData }

function ByteToHexStr(Data: Pointer; Len: Integer): String;
var
  I, Octets, PartOctets: Integer;
  DumpData: String;
begin
  if Len = 0 then Exit;
  I := 0;
  Octets := 0;
  PartOctets := 0;
  Result := '';
  while I < Len do
  begin
    case PartOctets of
      0: Result := Result + Format('%.4d: ', [Octets]);
      9:
      begin
        Inc(Octets, 10);
        PartOctets := -1;
        Result := Result + '    ' + DumpData + sLineBreak;
        DumpData := '';
      end;
    else
      begin
        Result := Result + Format('%s ', [IntToHex(TByteArray(Data^)[I], 2)]);
        if TByteArray(Data^)[I] in [$19..$FF] then
          DumpData := DumpData + Chr(TByteArray(Data^)[I])
        else
          DumpData := DumpData + '.';
        Inc(I);
      end;
    end;
    Inc(PartOctets);
  end;
  if PartOctets <> 0 then
  begin
    PartOctets := (8 - Length(DumpData)) * 3;
    Inc(PartOctets, 4);
    Result := Result + StringOfChar(' ', PartOctets) +
      DumpData
  end;
end;

function WordToHexStr(Data: Pointer; Len: Integer): String;
var
  I, Octets, PartOctets: Integer;
  ByteCount: Byte;
  OutputValue: DWORD;
begin
  if Len = 0 then Exit;
  I := 0;
  Octets := 0;
  PartOctets := 0;
  Result := '';
  while I < Len do
  begin
    case PartOctets of
      0: Result := Result + Format('%.4d: ', [Octets]);
      5:
      begin
        PartOctets := -1;
        Inc(Octets, 10);
        Result := Result + sLineBreak;
      end
    else
      ByteCount := 0;
      OutputValue := 0;
      if I < Len then
      begin
        OutputValue := TByteArray(Data^)[I];
        Inc(ByteCount, 2);
      end;
      if I + 1 < Len then
      begin
        OutputValue := OutputValue  + (TByteArray(Data^)[I + 1] shl 8);
        Inc(ByteCount, 2);
      end;
      if I + 2 < Len then
      begin
        OutputValue := OutputValue  + (TByteArray(Data^)[I + 2] shl 16);
        Inc(ByteCount, 2);
      end;
      if I + 3 < Len then
      begin
        OutputValue := OutputValue  + (TByteArray(Data^)[I + 3] shl 24);
        Inc(ByteCount, 2);
      end;
      Result := Result + Format('%s ', [IntToHex(OutputValue, ByteCount)]);
      Inc(I, 4);
    end;
    Inc(PartOctets);
  end;
end;

procedure TdlgViewRecordData.btnCopyClick(Sender: TObject);
begin
  ActivateKeyboardLayout(EventLogDemoMainForm.RussianLayout, KLF_REORDER);
  Clipboard.AsText := Format(
    ' :	%s' + sLineBreak +
    ' :	%s' + sLineBreak +
    ' :	%s' + sLineBreak +
    ' :	%s' + sLineBreak +
    ': %s' + sLineBreak +
    ': %s' + sLineBreak +
    ': %s' + sLineBreak +
    ' %s:' + sLineBreak + 
    ':' + sLineBreak + '%s' + sLineBreak +
    ':' + sLineBreak + '%s' + sLineBreak,
    [
      lblTypeVal.Caption,
      lblSourceVal.Caption,
      lblCategryVal.Caption,
      lblCodeVal.Caption,
      lblDateVal.Caption,
      lblTimeVal.Caption,
      lblUserVal.Caption,
      lblCompVal.Caption,
      memDescription.Text,
      memData.Text
    ]);
end;

procedure TdlgViewRecordData.btnDownClick(Sender: TObject);
var
  CurIndex: Integer;
begin
  CurIndex := EventLogDemoMainForm.lvEvents.Selected.Index;
  if CurIndex = EventLogDemoMainForm.lvEvents.Items.Count - 1 then
  begin
    if Application.MessageBox('   . ' +
      '   ?', ' ',
      MB_YESNO or MB_ICONQUESTION) = IDNO then
      Exit
    else
      CurIndex := -1;
  end;
  Inc(CurIndex);
  EventLogDemoMainForm.lvEvents.Items[CurIndex].Selected := True;
  if EventLogDemoMainForm.EventLog.Read(EventLogDemoMainForm.BackupShowing,
    Integer(EventLogDemoMainForm.lvEvents.Selected.Data), FEventRecord) then
    ShowRecData(FEventRecord);
end;

procedure TdlgViewRecordData.btnUpClick(Sender: TObject);
var
  CurIndex: Integer;
begin
  CurIndex := EventLogDemoMainForm.lvEvents.Selected.Index;
  if CurIndex = 0 then
  begin
    if Application.MessageBox('   . ' +
      '   ?', ' ',
      MB_YESNO or MB_ICONQUESTION) = IDNO then
      Exit
    else
      CurIndex := EventLogDemoMainForm.lvEvents.Items.Count;
  end;
  Dec(CurIndex);
  EventLogDemoMainForm.lvEvents.Items[CurIndex].Selected := True;
  if EventLogDemoMainForm.EventLog.Read(EventLogDemoMainForm.BackupShowing,
    Integer(EventLogDemoMainForm.lvEvents.Selected.Data), FEventRecord) then
    ShowRecData(FEventRecord);
end;

procedure TdlgViewRecordData.rbByteClick(Sender: TObject);
begin
  memData.Text :=
    ByteToHexStr(FEventRecord.BinData, Length(FEventRecord.BinData));
end;

procedure TdlgViewRecordData.rbWordClick(Sender: TObject);
begin
  memData.Text :=
    WordToHexStr(FEventRecord.BinData, Length(FEventRecord.BinData));
end;

procedure TdlgViewRecordData.ShowRecData(EventRecord: TFWEventLogRecord);
var
  AccountAndUser: String;
  BinDataLength: Integer;
begin
  FEventRecord := EventRecord;
  lblDateVal.Caption := DateToStr(EventRecord.TimeWritten);
  lblTimeVal.Caption := TimeToStr(EventRecord.TimeWritten);
  lblTypeVal.Caption := TFWEventLog.EventTypeToString(EventRecord.EventType);
  AccountAndUser := EventRecord.Domain + '\' + EventRecord.Account;
  if AccountAndUser = '\' then AccountAndUser := 'N\A';
  lblUserVal.Caption := AccountAndUser;
  lblCompVal.Caption := EventRecord.ComputerName;
  lblSourceVal.Caption := EventRecord.SourceName;
  lblCategryVal.Caption := EventRecord.Category;
  lblCodeVal.Caption := IntToStr(EventRecord.EventID);
  memDescription.Text := EventRecord.Description;
  BinDataLength := Length(EventRecord.BinData);
  if BinDataLength > 0 then
  begin
    lblData.Enabled := True;
    memData.Enabled := True;
    rbByte.Enabled := True;
    rbWord.Enabled := True;
    if rbByte.Checked then
      memData.Text := ByteToHexStr(EventRecord.BinData, BinDataLength)
    else
      memData.Text := WordToHexStr(EventRecord.BinData, BinDataLength);
  end
  else
  begin
    lblData.Enabled := False;
    memData.Enabled := False;
    rbByte.Enabled := False;
    rbWord.Enabled := False;
    memData.Text := '';
  end;
end;

end.
